# -*- coding: utf-8 -*-
# Copyright © 2010–2011 Oleg «Eleidan» Kulik.

import gtk
from time import localtime, strftime, time

CLOUD_THIKNESS  = 1                     # thikness of cloud's rectangle
C_FRESH         = "#575"
C_FRESH         = "#000080"             # Navy
C_FRESH         = "#FF4500"             # Orangered
C_FRESH         = "#DA70D6"             # Orchid
C_NEW           = "#00FFFF"             # Aqua
C_ON_DISCUSS    = "#FFAAAA"             # Red
C_ON_CORRECTION = "#9400D3"
C_FIX_DISCUSS   = "#FFCCEE"
C_FIX_CLOSED    = "#98FB98"             # Palegreen
C_FALSE         = "#aac"
C_FALSE         = "#9400D3"
C_FALSE         = "#ADFF2F"
C_FALSE         = "#4B0082"             # Indigo
C_FALSE         = "#FF0000"             # LifghtRed
C_HIGHLIGHT     = "#000080"             # Red

class Cloud(object):
    """Provides cloud functionality."""

    ###########################################################################
    def __init__(self, r, page, created=None, updated=None):
        # Data initialization
        self.rect = None                # coords of a cloud
        self.highlight = False          # flag that indicates highlight mode
        self.new_cloud = True           # flag that indicates new cloud
        self.page = None                # page where cloud lives, integer
        self.author = "Bad Santa"       # author of a cloud
        self.remark = "Some bug here..."# text of remark to a cloud
        self.created = None             # UTC time of creation of a cloud
        self.updated = None             # UTC time of update of a cloud
        self.status = None              # status of a cloud, integer
        self.color = None               # color of a cloud, depends on status
        self.id    = None               # unique identifier
        self.comments = []              # list of comments of a cloud
            #0:{"text": "", "author": "", "created": None}

        # Fetching data from params
        self.rect = r
        self.page = int(page+1)
        # Set time of creation
        if created is None:
            self.created = time()
        else:
            self.created = float(created)
        # Update time, it will be used to determine which cloud is newer
        if updated is None:
            self.updated = float(self.created)
        else:
            self.updated = updated
        
        self.get_color()

    ###########################################################################
    def set_status(self, st):
        """Set status of a cloud."""
        # If status is equal to current we have nothing to do here
        # NOTE: We do have to quit. Otherwise we will have an cloud.updated
        # changed.
        if self.status == st:
            return
        self.status = st
        # Update time, it will be used to determine which cloud is newer
        self.updated = time()
        self.get_color()

    ###########################################################################
    def import_status(self, st):
        """Import status of a cloud."""
        self.status = st
        self.get_color()

    ###########################################################################
    def set_remark(self, remark, author):
        """Set remark attributes."""
        self.author = author
        self.remark = remark
        # Now cloud is not only an "attempt"
        self.new_cloud = False
        self.set_status(0)
        # Create unique identifier
        self.id = author + str(self.rect) + str(self.created)
        return self

    ###########################################################################
    def import_remark(self, remark, author):
        """Import remark attributes."""
        self.author = author
        self.remark = remark
        # Now cloud is not only an "attempt"
        self.new_cloud = False
        # Create unique identifier
        self.id = author + str(self.rect) + str(self.created)
        return self

    ###########################################################################
    def add_comment(self, text, author, created=None):
        """Add a comment."""
        rec = {"text": u"", "author": u"", "created": float(0)}
        rec["author"] = author
        rec["text"] = text
        if created is None:
            rec["created"] = time()
            self.updated = time()
        else:
            rec["created"] = float(created)

        self.comments.append(rec)
        # Update time, it will be used to determine which cloud is newer
        return self

    ###########################################################################
    def draw(self, cr, scale=1):
        """Draw a rectangle with alpha channel."""
        color = self.color
        # Set line width
        cr.set_line_width(CLOUD_THIKNESS)
        # Set color from float values
        cr.set_source_rgba(color.red_float,
                color.green_float,
                color.blue_float,
                0.3)
        # Draw rectangle and fill it
        cr.rectangle(int(self.rect.x * scale),
                int(self.rect.y * scale),
                int(self.rect.width * scale),
                int(self.rect.height * scale))
        cr.fill()

        # Set color from float values
        cr.set_source_rgba(color.red_float,
                color.green_float,
                color.blue_float,
                1.0)
        # Draw frame
        cr.rectangle(int(self.rect.x * scale),
                int(self.rect.y * scale),
                int(self.rect.width * scale),
                int(self.rect.height * scale))
        cr.stroke()
        # Draw dashed frame if highlight mode is set to True
        if self.highlight:
            color = gtk.gdk.color_parse(C_HIGHLIGHT)
            # Set line width
            cr.set_line_width(CLOUD_THIKNESS)
            # Set line dash: list sets pairs of filled and blank in pixels
            cr.set_dash([5, 3])
            # Set color from float values
            cr.set_source_rgba(color.red_float,
                    color.green_float,
                    color.blue_float,
                    1.0)
            # Draw frame
            cr.rectangle(int(self.rect.x * scale),
                    int(self.rect.y * scale),
                    int(self.rect.width * scale),
                    int(self.rect.height * scale))
            cr.stroke()
            # Unset line dash
            cr.set_dash([])
        return self

    ###########################################################################
    def get_color(self):
        """Set proper color according to status."""
        # Color of a cloud depends on its status:
        if self.new_cloud:              # Just only drawn by user
            color = gtk.gdk.color_parse(C_FRESH)
        elif self.status == 0:          # New
            color = gtk.gdk.color_parse(C_NEW)
        elif self.status == 1:          # On discuss
            color = gtk.gdk.color_parse(C_ON_DISCUSS)
        elif self.status == 2:          # On correction
            color = gtk.gdk.color_parse(C_ON_CORRECTION)
        elif self.status == 3:          # Fixed and on discuss
            color = gtk.gdk.color_parse(C_FIX_DISCUSS)
        elif self.status == 4:          # Fixed and closed
            color = gtk.gdk.color_parse(C_FIX_CLOSED)
        else:                           # False remark, nothing to correct
            color = gtk.gdk.color_parse(C_FALSE)
        self.color = color
        return self

# end of cloud.py
