#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
Custom widgets
"""
import gtk
import gettext

from gettext import gettext as _
gettext.textdomain('hannibal')

class MySpinTool (gtk.ToolItem):

    def __init__(self):
        gtk.ToolItem.__init__(self)
        self.sb = gtk.SpinButton(adjustment=None, climb_rate=1, digits=0)
        self.add(self.sb)


class MyComboBox (gtk.ToolItem):

    def __init__(self):
        gtk.ToolItem.__init__(self)
        self.cb = gtk.combo_box_new_text()

        self.set_border_width(4)
        self.set_expand(False)
        self.set_homogeneous(True)
        self.cb.show()
        self.add(self.cb)


class RemarkGrabber (gtk.Dialog):

    def __init__(self, parent, cloud):
        gtk.Dialog.__init__(self, None, parent.window, gtk.DIALOG_NO_SEPARATOR)
        self.cloud = cloud
        self.par = parent
        #self.set_position(gtk.WIN_POS_MOUSE)
        self.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
        self.set_modal(True)
        self.set_decorated(False)
        self.vbox.set_border_width(12)
        self.action_area.set_border_width(12)
        #self.set_border_width(12)
        self.modify_bg(gtk.STATE_NORMAL, gtk.gdk.Color(6400, 6400, 6440))
        self.set_has_separator(False)
        #self.add_events(gtk.gdk.KEY_PRESS)
        #self.connect("key-release-event", self.key_release_event_cb)
        

        # Text entry for user's remark
        self.entry = gtk.Entry()
        # Set inner text alignment: left
        self.entry.set_alignment(0)
        # Set inner text max length
        #self.entry.set_max_length(100)
        # Set entry's max length in chars
        self.entry.set_width_chars(50)
        #entry.add_events(gtk.gdk.KEY_RELEASE)
        #entry.connect("key-released", self.on_key)
        self.entry.set_text("")
        self.entry.connect("activate", self.on_activate)

        self.vbox.pack_start(self.entry, False, False, 0)
        self.show_all()

    def on_activate(self, widget):
        text = widget.get_text()
        if text:
            self.cloud.set_remark(widget.get_text(), self.par.user)
            self.response(gtk.RESPONSE_OK)
            self.par.rem.add_cloud(self.cloud)
        else:
            self.response(gtk.RESPONSE_CANCEL)
        
        #self.par.view.queue_draw()
#        self.par.view.queue_draw_area(self.cloud.rect.x, self.cloud.rect.y,
#                self.cloud.rect.width, self.cloud.rect.height)

class Preferences(gtk.Dialog):
    def __init__(self, par, defs=None):
        gtk.Dialog.__init__(self)
        self.set_title(_('Hannibal Preferences'))
        self.set_resizable(False)
        self.set_property('skip-taskbar-hint', True)
        self.set_icon(par.window.get_icon())
        self.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)
        self.set_modal(True)
        self.set_has_separator(False)
        #self.set_size_request(150, 150)
            

        # Create a complex layout with gtk.Table.
        # One main table packs all the others.
        # Each cell of main table consist of child table,
        # which represents a group of widgets.
        # Main table: 2 rows, 2 cols, childs' sizes are not the same
        table_main = gtk.Table(2, 1, False)
        table_main.set_border_width(12)
        table_main.set_row_spacings(18)
        table_main.set_col_spacings(18)
        # "User info" table: 3 rows, 2 cols, childs' sizes are not the same
        table_user_info = gtk.Table(3, 2, False)
        # Set space between rows to 6 pixel, as HIG says
        table_user_info.set_row_spacings(6)

        # Label for "User info" group title text
        label_user_info = gtk.Label()
        # Set marked up text which allows to use bold style etc.
        label_user_info.set_markup(_('<b>User info</b>'))
        # Set alignment: left and middle
        label_user_info.set_alignment(xalign=0, yalign=0.5)

        # Label for "Name:" text
        label_user_name = gtk.Label(_('N_ame:'))
        # Allow mnemonics
        label_user_name.set_use_underline(True)
        # Set alignment: left and middle
        label_user_name.set_alignment(0, 0.5)
        # Text entry for user's name
        self.entry_user_name = gtk.Entry()
        # Bind with proper label
        label_user_name.set_mnemonic_widget(self.entry_user_name)
        # Set inner text alignment: left
        self.entry_user_name.set_alignment(0)
        # Set inner text max length
        self.entry_user_name.set_max_length(30)
        # Set entry's max length in chars
        self.entry_user_name.set_width_chars(20)
        # Pack entry into gtk.Alignment and justify it to the right
        align_user_name = gtk.Alignment(1.0, 0.5)
        align_user_name.add(self.entry_user_name)

        
        # Label for "Email:" text
        label_user_email = gtk.Label(_('_Email:'))
        label_user_email.set_use_underline(True)
        label_user_email.set_alignment(xalign=0, yalign=0.5)
        self.entry_user_email = gtk.Entry()
        label_user_email.set_mnemonic_widget(self.entry_user_email)
        self.entry_user_email.set_alignment(0)
        self.entry_user_email.set_max_length(30)
        self.entry_user_email.set_width_chars(20)
        align_user_email = gtk.Alignment(1.0, 0.5)
        align_user_email.add(self.entry_user_email)
        
        table_user_info.attach(label_user_info, 0, 2, 0, 1, gtk.FILL, 0)

        table_user_info.attach(label_user_name, 0, 1, 1, 2, gtk.FILL, 0, 12)
        table_user_info.attach(align_user_name, 1, 2, 1, 2, gtk.FILL | gtk.EXPAND, 0)

        table_user_info.attach(label_user_email, 0, 1, 2, 3, gtk.FILL, 0, 12)
        table_user_info.attach(align_user_email, 1, 2, 2, 3, gtk.FILL | gtk.EXPAND, 0)

        # Pack all tables with groups of widgets into main table
        table_main.attach(table_user_info, 0, 1, 0, 1, gtk.FILL | gtk.EXPAND, 0)

        self.vbox.pack_start(table_main, False, False, 0)
        if defs is not None:
            if "name" in defs:
                self.entry_user_name.set_text(defs["name"])
            if "email" in defs:
                self.entry_user_email.set_text(defs["email"])

        self.show_all()

# end of customs.py
