#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
Main module
"""
import os
import os.path
import gtk
import gtk.glade
import gettext

import view
import remarker
import customs
import client
import cloud
import time

#from time import gmtime, strftime

from gettext import gettext as _
gettext.textdomain('hannibal')

# Version info
VERSION = '0.1.3'

# Create preferences directory in user's homedir
PREF_DIR = os.path.expanduser("~/.hannibal")
if not os.path.exists(PREF_DIR):
    os.mkdir(PREF_DIR)
CONFIG = os.path.join(PREF_DIR, ".config")

# Set default path for some dialogs
OPEN_DIR = os.environ['HOME']
IMPORT_DIR = OPEN_DIR
EXPORT_DIR = OPEN_DIR

class Base(object):

    def __init__(self):
        self.actions = dict()
        self.groups = dict()
        self.toolbar_visible = True     # toolbar visibility
        self.side_panel_visible = True  # right panel visibility
        self.file_uri = ''              # path to PDF
        self.user = ''                  # user name
        self.email = ''
        self.document_title = None      # document title from meta
        self.document_created = None    # document creation timestamp
        self.document_updated = None    # document update timestamp

        # Generate ActionGroups and store them in the dictionary
        g = gtk.ActionGroup('Remarks')
        self.groups[g.get_name()] = g
        g = gtk.ActionGroup('View')
        self.groups[g.get_name()] = g
        g = gtk.ActionGroup('Go')
        self.groups[g.get_name()] = g


        # Fetch UI from XML
        self.window = gtk.Window(type=gtk.WINDOW_TOPLEVEL)
        self.window.set_title(_("Hannibal"))
        self.window.connect('delete-event', self.on_delete)
        self.window.connect('destroy', self.on_destroy)
        w = gtk.gdk.screen_width()
        h = gtk.gdk.screen_height()
        #self.window.set_size_request(int(w/2), h-100)
        self.window.resize(int(w/2), h-100)
        self.window.set_position(gtk.WIN_POS_CENTER)
        self.window.set_icon_from_file('icon.png')

        # Create the Layout of general parts of UI
        self.vbox = gtk.VBox(homogeneous=False, spacing=0)
        self.menubar = gtk.MenuBar()
        self.toolbar = gtk.Toolbar()
        self.statusbar = gtk.Statusbar()
        self.hpaned = gtk.HPaned()
        self.view_sw = gtk.ScrolledWindow()
        self.view_sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)

        # Custom widgets
        self.view = view.View(self)
        self.rem = remarker.Remarker(self)

        # Packing
        view_alignment = gtk.Alignment(xalign=0.5, yalign=0.5)
        view_alignment.add(self.view)
        self.view_sw.add_with_viewport(view_alignment)
        self.hpaned.pack1(self.view_sw, resize=True, shrink=False)
        self.hpaned.pack2(self.rem, resize=False, shrink=False)
        self.vbox.pack_start(self.menubar, expand=False, fill=False, padding=0)
        self.vbox.pack_start(self.toolbar, expand=False, fill=False, padding=0)
        self.vbox.pack_start(self.hpaned, expand=True, fill=True, padding=0)
        self.window.add(self.vbox)

        # Create AccelGroup for keybindings storing
        accel_group = gtk.AccelGroup()
        self.window.add_accel_group(accel_group)

        # Create Menu
        # MenuBar is a toplevel container for menu items
        # Add Menu as a child container for File menu item

        #######################################################################
        # File
        menu_item = gtk.MenuItem(_('_File'))
        # Add Menu as a child container for File menu item
        menu = gtk.Menu()
        menu_item.set_submenu(menu)
        self.menubar.append(menu_item)


        # File->Open
        # Create menu item with given acceleration group
        mi = gtk.ImageMenuItem(gtk.STOCK_OPEN, accel_group)
        # Connect action to the callback
        mi.connect('activate', self.on_open)
        # Append menu item to the menu
        menu.append(mi)

        # File->Import
        # Create Action
        a = gtk.Action('import', _('_Import remarks...'), _('Import locally stored remarks'), None)
        # Connect action to the callback
        a.connect('activate', self.on_import)
        # Set AccelGroup for the action
        self.groups['Remarks'].add_action_with_accel(a, None)
        a.set_accel_group(accel_group)
        a.connect_accelerator()
        # Create menu item widget from the action
        mi = a.create_menu_item()
        # Append menu item to the menu
        menu.append(mi)
        self.actions[a.get_name()] = a

        # File->Export
        # Create Action
        a = gtk.Action('export', _('_Export remarks...'), _('Save remarks locally'), None)
        # Connect action to the callback
        a.connect('activate', self.on_export)
        # Set AccelGroup for the action
        self.groups['Remarks'].add_action_with_accel(a, None)
        a.set_accel_group(accel_group)
        a.connect_accelerator()
        # Create menu item widget from the action
        mi = a.create_menu_item()
        # Append menu item to the menu
        menu.append(mi)
        self.actions[a.get_name()] = a

        # Create separator and append it to the menu
        sep = gtk.SeparatorMenuItem()
        menu.append(sep)

        # File->Quit
        # Create Action
        mi = gtk.ImageMenuItem(gtk.STOCK_QUIT, accel_group)
        # Connect action to the callback
        mi.connect('activate', self.on_quit)
        # Append menu item to the menu
        menu.append(mi)


        #######################################################################
        # Edit
        menu_item = gtk.MenuItem(_('_Edit'))
        # Add Menu as a child container for File menu item
        menu = gtk.Menu()
        menu_item.set_submenu(menu)
        self.menubar.append(menu_item)

        # Edit->Preferences
        # Create Action
        mi = gtk.ImageMenuItem(gtk.STOCK_PREFERENCES, accel_group)
        # Connect action to the callback
        mi.connect('activate', self.on_preferences)
        # Append menu item to the menu
        menu.append(mi)


        #######################################################################
        # View
        menu_item = gtk.MenuItem(_('_View'))
        # Add Menu as a child container for File menu item
        menu = gtk.Menu()
        menu_item.set_submenu(menu)
        self.menubar.append(menu_item)

        # View->Toolbar
        # Create Action
        mi = gtk.CheckMenuItem(_('_Toolbar'), accel_group)
        # Connect action to the callback
        mi.connect('activate', self.on_toolbar_toggled)
        # Set state of the CheckMenuItem
        mi.set_active(self.toolbar_visible)
        # Append menu item to the menu
        menu.append(mi)

        # View->Side panel
        # Create Action
        a = gtk.ToggleAction('sidepane', _('_Side panel'), _('Toggle side panel'), None)
        # Connect action to the callback
        a.connect('activate', self.on_side_panel_toggled)
        # Set AccelGroup for the action
        self.groups['View'].add_action_with_accel(a, 'F9')
        a.set_accel_group(accel_group)
        a.connect_accelerator()
        # Create menu item widget from the action
        mi = a.create_menu_item()
        # Set state of the CheckMenuItem
        mi.set_active(self.side_panel_visible)
        # Append menu item to the menu
        menu.append(mi)
        self.actions[a.get_name()] = a


        #######################################################################
        # Go
        menu_item = gtk.MenuItem(_('_Go'))
        # Add Menu as a child container for File menu item
        menu = gtk.Menu()
        menu_item.set_submenu(menu)
        self.menubar.append(menu_item)

        # Go->Previous
        # Create Action
        a = gtk.Action('Previous', None, _('Go to the previous page'), gtk.STOCK_GO_UP)
        # Connect action to the callback
        a.connect('activate', self.on_page_prev)
        # Set AccelGroup for the action
        self.groups['Go'].add_action_with_accel(a, '<Control>Page_Up')
        a.set_accel_group(accel_group)
        a.connect_accelerator()
        # Create menu item widget from the action
        mi = a.create_menu_item()
        # Append menu item to the menu
        menu.append(mi)
        self.actions[a.get_name()] = a

        # Go->Previous
        # Create Action
        a = gtk.Action('Next', None, _('Go to the next page'), gtk.STOCK_GO_DOWN)
        # Connect action to the callback
        a.connect('activate', self.on_page_next)
        # Set AccelGroup for the action
        self.groups['Go'].add_action_with_accel(a, '<Control>Page_Down')
        a.set_accel_group(accel_group)
        a.connect_accelerator()
        # Create menu item widget from the action
        mi = a.create_menu_item()
        # Append menu item to the menu
        menu.append(mi)
        self.actions[a.get_name()] = a


        #######################################################################
        # Help
        menu_item = gtk.MenuItem(_('_Help'))
        # Add Menu as a child container for File menu item
        menu = gtk.Menu()
        menu_item.set_submenu(menu)
        self.menubar.append(menu_item)

        # Help->Toolbar
        # Create Action
        mi = gtk.ImageMenuItem(gtk.STOCK_ABOUT, accel_group)
        # Connect action to the callback
        mi.connect('activate', self.on_about)
        # Append menu item to the menu
        menu.append(mi)



        # Create toolbar item widgets from the actions
        do_next = self.actions['Next'].create_tool_item()
        do_next.set_is_important(True)
        do_prev = self.actions['Previous'].create_tool_item()
        do_prev.set_is_important(True)

        # Pack all widgets into the toolbar
        self.toolbar.add(do_next)
        self.toolbar.add(do_prev)
        #self.toolbar.insert(do_prev, 1)

        self.page_field = customs.MySpinTool()
        self.page_field.sb.connect("value-changed", self.on_page_change)

        self.scale_field = customs.MyComboBox()
        self.scale_field.cb.connect("changed", self.on_scale_change)
        #self.scale_field.cb.append_text("Уместить в окне")
        #self.scale_field.cb.append_text("По ширине страницы")
        self.scale_field.cb.append_text("100%")
        self.scale_field.cb.append_text("200%")
        self.scale_field.cb.append_text("300%")
        self.scale_field.cb.set_active(0)


        self.toolbar.add(gtk.SeparatorToolItem())
        self.toolbar.add(self.page_field)
        self.toolbar.add(gtk.SeparatorToolItem())
        self.toolbar.add(self.scale_field)


        self.toolbar.set_sensitive(False)

        self.window.show_all()

        self.get_preferences()
        if not self.user:
            self.set_preferences()
        self.rem.get_clouds()
        self.groups['Remarks'].set_sensitive(False)
        self.groups['Go'].set_sensitive(False)

    ###########################################################################
    def open_file(self, f):
        """Load file with some checks."""
        if f is not None and os.path.isfile(f):
            self.view.get_from_file(f)
            self.page_field.sb.set_range(1, self.view.n_pages)
            self.page_field.sb.set_increments(1, 1)
            # Prepare some widgets for new document
            self.page_field.sb.set_value(1)
            # Some widgets should be sensitive now
            self.groups['Remarks'].set_sensitive(True)
            self.groups['Go'].set_sensitive(True)

    ###########################################################################
    def get_preferences(self):
        """Get preferences."""
        try:
            # Try to get data from config file
            f = open(CONFIG, "r")
            self.user, self.email = eval(f.read())
            f.close()
        except:
            print "\t[w] Cannot open a file"

    ###########################################################################
    def set_preferences(self):
        """Set preferences."""
        p = customs.Preferences(self, {"name":self.user, "email":self.email})
        r = p.run()
        if r == gtk.RESPONSE_CLOSE:
            name = p.entry_user_name.get_text()
            email = p.entry_user_email.get_text()
            if name:
                self.user = name
                self.email = ''.join(email)
                try:
                    f = open(CONFIG, "w")
                    f.write(str([self.user, self.email]))
                    f.close()
                except:
                    print "\t[e] Cannot save prefs."
        p.destroy()


    ###########################################################################
    # Callback methods
    ###########################################################################

    ###########################################################################
    def on_realize(self, event):
        """Called when "realized" signal was received."""
        self.gdkw = self.dwg.window
        self.gc = self.gdkw.new_gc()

    ###########################################################################
    def on_expose(self, widget, event):
        """Called when "expose" signal was received.

        Usually it is raised when widget should be visually refreshed.
        """
        cr = widget.window.cairo_create()
        cr.set_source_rgb(1, 1, 1)

        if self.scale != 1:
            cr.scale(self.scale, self.scale)

        if self.buf is not None:
            self.get_buffer()

        self.put_buffer()

    ###########################################################################
    def on_toolbar_toggled(self, widget):
        """Called when toolbar visibility is toggled.

        It should be connected to proper menu item.
        """
        # Get new value
        self.toolbar_visible = widget.get_active()

        # If True — toolbar is visible
        if self.toolbar_visible:
            # Check whether toolbar has a parent (is it inside any container?).
            # If not — pack it into container at the 1st position.
            if not self.toolbar.get_parent():
                self.vbox.pack_start(self.toolbar,
                                    expand=False,
                                    fill=True,
                                    padding=0)
                self.vbox.reorder_child(child=self.toolbar, position=1)

        # Else if toolbar is already in container it has to be hidden.
        # It is done by removing toolber from container.
        elif self.toolbar.get_parent():
            self.vbox.remove(self.toolbar)

    ###########################################################################
    def on_side_panel_toggled(self, widget):
        """Called when side panel visibility is toggled.

        It should be connected to proper menu item.
        """
        # Get new value
        self.side_panel_visible = widget.get_active()

        # If True — side panel is visible
        if self.side_panel_visible:
            # Check whether panel has a parent (is it inside any container?).
            # If not — pack it into container at the 1st position.
            if not self.rem.get_parent():
                self.hpaned.pack2(self.rem, resize=False, shrink=True)

        # Else if panel is already in container it has to be hidden.
        # It is done by removing panel from container.
        elif self.rem.get_parent():
            self.hpaned.remove(self.rem)

    ###########################################################################
    def on_scale_change(self, widget):
        """Called on scale change."""
        self.view.set_scale(self.scale_field.cb.get_active())

    ###########################################################################
    def on_page_change(self, widget):
        """Called on page change."""
        cp = widget.get_value_as_int()
        self.view.set_page_num(cp-1)

        if self.view.n_pages > cp:
            self.actions['Next'].set_sensitive(True)
        else:
            self.actions['Next'].set_sensitive(False)

        if cp <= 1:
            self.actions['Previous'].set_sensitive(False)
        else:
            self.actions['Previous'].set_sensitive(True)

    ###########################################################################
    def on_page_next(self, widget):
        """Callback for "Next" toolbutton clicked.

        Page number should be incremented.
        """
        self.view.next_page()
        self.page_field.sb.spin(gtk.SPIN_STEP_FORWARD)

    ###########################################################################
    def on_page_prev(self, widget):
        """Callback for "Prev" toolbutton clicked.

        Page number should be decremented.
        """
        self.view.prev_page()
        self.page_field.sb.spin(gtk.SPIN_STEP_BACKWARD)

    ###########################################################################
    def agree_to_overwrite(self, f, par):
        if os.path.exists(f) and os.path.isfile(f):
            dlg = gtk.MessageDialog(parent=par,
                                    flags=gtk.DIALOG_MODAL,
                                    type=gtk.MESSAGE_QUESTION,
                                    buttons=gtk.BUTTONS_YES_NO,
                                    message_format=None)
            dlg.set_icon(self.window.get_icon())
            the_file = os.path.basename(f)
            dlg.set_markup(_("<b>File '{0}' already exists. Overwrite it?</b>").format(the_file))
            dlg.format_secondary_text(_("The file already exists in current location.\nReplacing it will overwrite its contents."))
            r = dlg.run()
            dlg.destroy()
            if r == gtk.RESPONSE_YES:
                return True
            if r == gtk.RESPONSE_NO:
                return False
        return True

    ###########################################################################
    def on_export(self, widget):
        global EXPORT_DIR
        """Save remarks to file."""
        dlg = gtk.FileChooserDialog(title=None,
                                    parent=self.window,
                                    action=gtk.FILE_CHOOSER_ACTION_SAVE,
                                    buttons=None,
                                    backend=None)
        dlg.set_icon(self.window.get_icon())
        dlg.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
        dlg.add_button(gtk.STOCK_SAVE, 1983)

        dlg.set_icon(self.window.get_icon())
        dlg.set_title(_("Export Remarks..."))
        dlg.set_default_response(1983)
        # Set folder to what we think is preferred
        dlg.set_current_folder(EXPORT_DIR)

        # Add FileFilter: allows to filter file types to be opened
        filt = gtk.FileFilter()
        #filt.add_mime_type("application/pdf")
        filt.set_name("*.hrx")
        filt.add_pattern("*.hrx")
        dlg.add_filter(filt)

        filt = gtk.FileFilter()
        filt.set_name("*.*")
        filt.add_pattern("*.*")
        dlg.add_filter(filt)
        dlg.set_current_name('remarks_'+self.user+'.hrx')

        done = False
        while not done:
            res = dlg.run()

            folder = dlg.get_current_folder()
            if folder:
                EXPORT_DIR = folder

            if res == gtk.RESPONSE_CANCEL:
                done = True
            elif res == gtk.RESPONSE_DELETE_EVENT:
                done = True
            elif res == 1983:
                f = dlg.get_filename()
                if f is not None and self.agree_to_overwrite(f, dlg):
                    self.rem.export_to_hrx(f)
                    done = True
        dlg.destroy()

    ###########################################################################
    def on_import(self, widget):
        """Get file for import."""
        global IMPORT_DIR
        dlg = gtk.FileChooserDialog(title=None,
                                    parent=self.window,
                                    action=gtk.FILE_CHOOSER_ACTION_OPEN,
                                    buttons=None,
                                    backend=None)
        dlg.set_icon(self.window.get_icon())
        dlg.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
        dlg.add_button(gtk.STOCK_OPEN, 1983)

        dlg.set_icon(self.window.get_icon())
        dlg.set_title(_("Import Remarks..."))
        dlg.set_default_response(1983)
        # Set folder to what we think is preferred
        dlg.set_current_folder(IMPORT_DIR)

        # Add FileFilter that allows to filter file types to be opened
        filt = gtk.FileFilter()
        #filt.add_mime_type("application/pdf")
        filt.set_name("*.hrx")
        filt.add_pattern("*.hrx")
        dlg.add_filter(filt)
        filt = gtk.FileFilter()
        filt.set_name("*.*")
        filt.add_pattern("*.*")
        dlg.add_filter(filt)

        res = dlg.run()
        f = dlg.get_filename()

        folder = dlg.get_current_folder()
        if folder:
            IMPORT_DIR = folder

        dlg.destroy()

        if res == gtk.RESPONSE_CANCEL:
            pass
        elif res == gtk.RESPONSE_DELETE_EVENT:
            pass
        elif res == 1983:
            if f is not None and os.path.isfile(f):
#                try:
                    self.rem.import_from_hrx(f)
                    self.rem.cloud_list.set_cursor((0,))
#                except:
#                    print "[e] Can't open a file!"

    ###########################################################################
    def on_refresh(self, widget):
        """Callback for "Refresh" toolbutton clicked.

        It should perform some requests to server.
        """
        #TODO: implement in case of using client/server functionality
        pass

    ###########################################################################
    def on_open(self, widget):
        """Callback for "File->Open" activation."""
        global OPEN_DIR
        dlg = gtk.FileChooserDialog(title=None,
                                    parent=self.window,
                                    action=gtk.FILE_CHOOSER_ACTION_OPEN,
                                    buttons=None,
                                    backend=None)
        dlg.set_icon(self.window.get_icon())
        dlg.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
        dlg.add_button(gtk.STOCK_OPEN, 1983)

        dlg.set_icon(self.window.get_icon())
        dlg.set_title(_("Open File"))
        dlg.set_default_response(1983)
        # Set folder to what we think is preferred
        dlg.set_current_folder(OPEN_DIR)

        filt = gtk.FileFilter()
        filt.add_mime_type("application/pdf")
        filt.set_name("*.pdf")
        dlg.add_filter(filt)

        res = dlg.run()

        if res == gtk.RESPONSE_CANCEL:
            pass
        elif res == gtk.RESPONSE_DELETE_EVENT:
            pass
        elif res == 1983:
            f = dlg.get_filename()
            # FIXME: CHECK must be done!!!
            try:
                self.open_file(f)
            except:
                print "[e] Can't open a file!"

        folder = dlg.get_current_folder()
        if folder:
            OPEN_DIR = folder

        dlg.destroy()

    ###########################################################################
    def on_preferences(self, widget):
        """Callback for "Edit->Preferences" activation."""
        self.set_preferences()

    ###########################################################################
    def on_about(self, widget):
        """Callback for "Help->About" activation."""
        global VERSION
        dlg = gtk.AboutDialog()
        dlg.set_icon(self.window.get_icon())
        dlg.set_program_name('Hannibal')
        dlg.set_name('Hannibal')
        dlg.set_comments('Hannibal is a multi-user PDF proofreading application.')
        dlg.set_version(VERSION)
        dlg.set_property('skip-taskbar-hint', True)
        dlg.set_license('''
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License with
the Debian GNU/Linux distribution in file /usr/share/common-licenses/GPL;
if not, write to the Free Software Foundation, Inc., 59 Temple Place,
Suite 330, Boston, MA  02111-1307  USA

On Debian systems, the complete text of the GNU General Public
License, version 3, can be found in /usr/share/common-licenses/GPL-3.
        ''')
        dlg.set_wrap_license(True)
        authors = ['Oleg «Eleidan» Kulik <helh-saintman@yandex.ru>',
                   'Valentina Mukhamedzhanova',
                   'Victor Nikulshin\n',

                   'Contributors:',
                   'Alexandr Markevich',
                   'Lyubov Kasyanova',
                   'Maxim Kalinin',
                   'Oleg Anisimov']
        dlg.set_authors(authors)
        copyright = u"Copyright © 2010–2011 Oleg «Eleidan» Kulik\n"
        copyright += u"Copyright © 2010 Valentina Mukhamedzhanova"
        dlg.set_copyright(copyright)
        dlg.run()
        dlg.destroy()

    ###########################################################################
    def check_for_unsaved(self):
        if self.rem.is_changed:
            dlg = gtk.MessageDialog(parent=None,
                                    flags=0,
                                    type=gtk.MESSAGE_WARNING,
                                    buttons=gtk.BUTTONS_YES_NO,
                                    message_format=None)
            dlg.set_icon(self.window.get_icon())
            dlg.set_markup(_("<b>There are some unsaved changes.</b>"))
            dlg.format_secondary_text(_("Quit anyway?"))
            r = dlg.run()
            dlg.destroy()
            if r == gtk.RESPONSE_YES:
                return True
            if r == gtk.RESPONSE_NO:
                return False
        return True

    ###########################################################################
    def on_delete(self, widget, event, data=None):
        """Close application and its window."""
        if self.check_for_unsaved():
            # Do not close window
            return False
        # Let the window be closed and emit 'destroy' signal
        return True

    ###########################################################################
    def on_quit(self, widget, data=None):
        if self.check_for_unsaved():
            gtk.main_quit()

    ###########################################################################
    def on_destroy(self, widget):
        gtk.main_quit()

    ###########################################################################
    def run(self):
        gtk.main()

if __name__ == "__main__":
    base = Base()
    base.run()

# end of main.py
