#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
Widget that provides remarks functionality
"""

import gtk
import pango
import gettext

import xml.etree.ElementTree as ET

import cloud

from time import localtime, strftime, time

from gettext import gettext as _
gettext.textdomain('hannibal')

#import cloudlist

###############################################################################
class Remarker(gtk.VPaned):
    """Provides functionality of right pane."""
    def __init__(self, parent):
        gtk.VPaned.__init__(self)
        self.par = parent               # parent widget
        self.is_changed = False         # flag that for changes in clouds
        #self.current_row = None         # current (selected) row in list
        self.current_iter = None        # current (selected) iter of the list
        self.cloud_list = CloudList(self)
        self.cloud_sw = gtk.ScrolledWindow()
        #self.cloud_sw = gtk.ScrolledWindow(hadjustment=None, vadjustment=None)
        self.cloud_sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.cloud_sw.add(self.cloud_list)
        self.pack1(self.cloud_sw, resize=True, shrink=False)
        #self.pack1(self.cloud_list, resize=True, shrink=False)

        self.vbox = gtk.VBox(homogeneous=False, spacing=0)


        # Create TextView for list of cloud's comments
        self.comments = Comments()
        # Create ScrolledWindow for TextView as it has no own one
        self.comments_sw = gtk.ScrolledWindow()
        self.comments_sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.comments_sw.add_with_viewport(self.comments)

        # Create TextView for new comment
        self.new_comment = NewComment()
        # Create ScrolledWindow for TextView as it has no own one
        self.new_comment_sw = gtk.ScrolledWindow()
        self.new_comment_sw.set_policy(gtk.POLICY_AUTOMATIC,
                                       gtk.POLICY_AUTOMATIC)
        self.new_comment_sw.add_with_viewport(self.new_comment)

        # Create button for adding comments
        self.add_comment_button = gtk.Button(stock=gtk.STOCK_ADD)
        self.add_comment_button.set_focus_on_click(False)
        self.add_comment_button.connect("clicked", self.on_add_clicked)

        # Create widget with a set of controls for cloud manipulations
        self.cloud_controls = CloudControls(self)


        self.vbox.pack_start(self.cloud_controls, expand=False, fill=False)

        self.vbox.pack_start(self.comments_sw, expand=True, fill=True)
        self.vbox.pack_end(self.add_comment_button, expand=False, fill=True)
        self.vbox.pack_end(self.new_comment_sw, expand=False, fill=True, padding=0)
        self.pack2(self.vbox, resize=True, shrink=False)
        # Set to sensitive=False because no cloude is selected by default
        self.add_comment_button.set_sensitive(False)
        self.cloud_controls.status.set_sensitive(False)

    ###########################################################################
    def add_cloud(self, cloud):
        """Add a cloud to the list."""
        self.cloud_list.add_cloud(cloud)
        self.is_changed = True

    ###########################################################################
    def get_clouds(self, page=None, author=None):
        """Get list of clouds."""
        clouds = []                     # list of clouds
        flag_author = False             # flag
        flag_page = False               # flag
        # Get clouds for given page number or author
        if page is not None or author is not None:
            # Get model where clouds are stored
            model = self.cloud_list.get_model()
            # Fetch all records
            for row in model:
                # Get value from first column, where cloud's object stored
                # and check if cloud belongs to given page
                if row[0].page == page:
                    # Ok, add cloud to the list
                    flag_page = True
                # Get value from first column, where cloud's object stored
                # and check if cloud belongs to given author
                if row[0].author == author:
                    # Ok, add cloud to the list
                    flag_author = True
                # Get value from first column, where cloud's object stored
                # and check if any cloud's comment belongs to given author
                for c in row[0].comments:
                    if c["author"] == author:
                        # Ok, add cloud to the list
                        flag_author = True
                # If every condition is met, add cloud to list
                if author is not None and page is not None:
                    if flag_page and flag_author:
                        clouds.append(row[0])
                elif author is not None and page is None:
                    if flag_author:
                        clouds.append(row[0])
                elif author is None and page is not None:
                    if flag_page:
                        clouds.append(row[0])
                flag_page = False
                flag_author = False
        # Get all clouds if no page was given
        else:
            # Get model where clouds are stored
            model = self.cloud_list.get_model()
            # Fetch all records
            for row in model:
                clouds.append(row[0])
        return clouds

    ###########################################################################
    # FIXME: It should be deprecated in future
    def add_clouds(self, clouds):
        for i in clouds:
            for c in clouds[i]:
                self.cloud_list.add_cloud(c)

    ###########################################################################
    def import_from_hrx(self, fname):
        """Import clouds from XML."""
        created = None
        updated = None
        doc_title = None
        doc_created = None
        doc_updated = None
        tree = ET.ElementTree()
        tree.parse(fname)
        root = tree.getroot()
        childs = root.getchildren()
        for child in childs:
            if child.tag == "user":
                #self.par.user = child.text
                pass
            elif child.tag == "document":
                doc_title = child.get("title")

                if doc_title != self.par.document_title:
                    dlg = gtk.MessageDialog(parent=None,
                                            flags=0,
                                            type=gtk.MESSAGE_WARNING,
                                            buttons=gtk.BUTTONS_YES_NO,
                                            message_format=None)
                    dlg.set_icon(self.par.window.get_icon())
                    dlg.set_markup(_("<b>Remarks you are trying to import\nis not for this document.</b>"))
                    dlg.format_secondary_text(_("Import anyway?"))
                    r = dlg.run()
                    dlg.destroy()
                    if r == gtk.RESPONSE_YES:
                        pass
                    if r == gtk.RESPONSE_NO:
                        return
            elif child.tag == "clouds":
                clouds = child.getchildren()
                for cl in clouds:
                    for item in cl:
                        
                        if item.tag == "rectangle":
                            r = gtk.gdk.Rectangle(int(item.get("x")),
                                                  int(item.get("y")),
                                                  int(item.get("width")),
                                                  int(item.get("height")))
                        if item.tag == "created":
                            created = float(item.text)
                        if item.tag == "updated":
                            updated = float(item.text)
                        if item.tag == "remark":
                            remark = item.text
                        if item.tag == "comments":
                            comms = list()
                            for cmnt in item:
                                rec = dict()
                                rec["author"] = cmnt.get("author")
                                rec["text"] = cmnt.text
                                rec["created"] = float(cmnt.get("created"))
                                comms.append(rec)
                                del rec
                    c = cloud.Cloud(r, int(cl.get("page")), created, updated)
                    c.import_remark(remark, cl.get("author"))
                    c.import_status(int(cl.get("status")))
                    for comm in comms:
                        c.add_comment(comm["text"],
                                      comm["author"],
                                      float(comm["created"]))
                    self.add_cloud(c)
        self.is_changed = True

    ###########################################################################
    def export_to_hrx(self, fname):
        """Export clouds into XML."""
        root = ET.Element("root")
        user = ET.SubElement(root, "user")
        user.text = self.par.user
        document = ET.SubElement(root, "document")
        document.attrib = {"title" : self.par.document_title}
        clouds = ET.SubElement(root, "clouds")

        clouds_list = self.get_clouds(author=self.par.user)
        for cl in clouds_list:
            cloud = ET.Element("cloud")
            cloud.attrib = {"author":cl.author,
                            "page":str(cl.page-1),
                            "status":str(cl.status)}

            created = ET.SubElement(cloud, "created")
            created.text = str(cl.created)
            updated = ET.SubElement(cloud, "updated")
            updated.text = str(cl.updated)
            remark = ET.SubElement(cloud, "remark")
            remark.text = cl.remark
            rectangle = ET.SubElement(cloud, "rectangle")
            rectangle.attrib = {"x":str(cl.rect.x),
                                "y":str(cl.rect.y),
                                "width":str(cl.rect.width),
                                "height":str(cl.rect.height)}
            comments = ET.SubElement(cloud, "comments")
            for c in cl.comments:
                comment = ET.SubElement(comments, "comment")
                comment.text = c["text"]
                c["created"] = str(c["created"])
                comment.attrib = {"created":c["created"],
                                  "author":c["author"]}
                    
            clouds.append(cloud)

        text = '﻿<?xml version="1.0" encoding="utf-8"?>\n'
        text += ET.tostring(root, "utf-8")
        f = open(fname, "w")
        f.write(text)
        f.close()
        self.is_changed = False

    ###########################################################################
    def clear(self):
        # Get model and clear its list
        self.cloud_list.get_model().clear()
        # Clear TextView for the comments
        self.comments.get_buffer().set_text('')
        
        return self

    ###########################################################################
    def on_add_clicked(self, widget):
        buf = self.new_comment.get_buffer()
        text = buf.get_text(*(buf.get_bounds()))
        text = text.strip()
        if not text:
            return
        # Get cursor position in the list
        path, _ = self.cloud_list.get_cursor()
        # Get cloud at cursor
        cloud = self.cloud_list.get_model()[path][0]
        cloud.add_comment(text, self.par.user)
        self.comments.compose_comments(cloud)
        self.new_comment.get_buffer().set_text('')
        self.is_changed = True


###############################################################################
class CloudControls(gtk.VBox):
    """Provides some widgets to maintain clouds' list.

    * Status of a cloud."""
    def __init__(self, parent):
        gtk.VBox.__init__(self)
        self.par = parent
        # Create simple combo box with text only
        self.status = gtk.combo_box_new_text()
        # Fill combo box with proper status items
        self.status.append_text(_("New"))
        self.status.append_text(_("On discuss"))
        self.status.append_text(_("On correction"))
        self.status.append_text(_("Fixed and is on discuss"))
        self.status.append_text(_("Fixed and closed"))
        self.status.append_text(_("False"))
        # Set up a tooltip for combo box
        self.status.set_has_tooltip(True)
        self.status.set_tooltip_text(_("Status of a cloud"))
        # Set up the default item to be selected: None
        self.status.set_active(-1)
        # Add combo box to the VBox
        self.pack_start(self.status, expand=False, fill=False)
        # Bind some events to their callbacks
        self.status.connect("changed", self.on_changed)

    def on_changed(self, widget):
        # Get path of record in tree
        path, _ = self.par.cloud_list.get_cursor()
        cloud = self.par.cloud_list.get_model()[path][0]
        cloud.set_status(widget.get_active())
        #TODO: Use this algorithm for fetching records from the list.
        # Use of global variables to store path and TreeIter
        # shoud be deprecated.

        # Get TreeIter of record in model
        tree_iter = self.par.cloud_list.get_model().get_iter(path)
        # Set color of a cloud in model
        self.par.cloud_list.get_model().set_value(tree_iter, 3, cloud.color)
        # Force redraw of view content
        self.par.par.view.queue_draw()
        self.is_changed = True


###############################################################################
class NewComment(gtk.TextView):
    """Widget for creation of new comments."""
    def __init__(self):
        # Create text buffer
        buf = gtk.TextBuffer()
        # Set text into the buffer
        #buf.set_text("Nothing Else Matters...")
        # Create wifget with prepared buffer
        gtk.TextView.__init__(self, buf)
        # Allow to edit a content of the widget's buffer
        self.set_editable(True)
        # Wrap mode for the widget's buffer
        self.set_wrap_mode(gtk.WRAP_WORD_CHAR)


###############################################################################
class Comments(gtk.TextView):
    """Widget for representation of cloud's comments."""
    def __init__(self):
        gtk.TextView.__init__(self)
        self.set_editable(False)
        self.set_wrap_mode(gtk.WRAP_WORD_CHAR)
        buf = self.get_buffer()
        color = gtk.gdk.color_parse("#AAA")
        buf.create_tag("date", foreground_gdk=color)
        buf.create_tag("author", weight=pango.WEIGHT_BOLD)
        #self.set_buffer(buf)

    ###########################################################################
    def compose_comments(self, cl):
        """Compose a remark with list of comments from a cloud."""
        buf = self.get_buffer()

        # Actually, it must be a remark left by a creator, let's show it
        text = cl.remark + '\n'         # text of remark from cloud's creator
        buf.set_text(text)
        pos = buf.get_end_iter()
        text = strftime("%Y-%m-%d %H:%M", localtime(cl.created)) + '\n'
        buf.insert_with_tags_by_name(pos, text, "date")
        pos = buf.get_end_iter()

        text = cl.author
        buf.insert_with_tags_by_name(pos, text, "author")
        pos = buf.get_end_iter()

        # If there is any comments we should add them to the list as well
        for c in cl.comments:
            text = '\n\n' + c["text"]   # text of a comment
            buf.insert(pos, text)
            pos = buf.get_end_iter()
            text = '\n' + strftime("%Y-%m-%d %H:%M", localtime(float(c["created"])))
            buf.insert_with_tags_by_name(pos, text, "date")
            pos = buf.get_end_iter()
            text = '\n' + c["author"]          # date + author's name of creation

            buf.insert_with_tags_by_name(pos, text, "author")
            pos = buf.get_end_iter()

###############################################################################
class CloudList(gtk.TreeView):
    """Widget that contain a list of all clouds."""
    def __init__(self, parent=None):
        gtk.TreeView.__init__(self)
        self.par = parent               # parent widget
        # Create model to maintain the list of clouds
        self.set_model(gtk.ListStore(object, int, str, gtk.gdk.Color))
        self.get_model().set_sort_column_id(1, gtk.SORT_ASCENDING)
        # Bind some events to their callbacks
        self.connect("row-activated", self.on_activated)
        self.connect("cursor-changed", self.on_cursor_changed)
        # Set rules for odd rows: make no difference
        self.set_rules_hint(False)
        # Bind some events to their callbacks
        self.create_columns(self)

    ###########################################################################
    def add_cloud(self, cloud):
        """Add a cloud to the list."""
        # Get model where clouds are stored
        model = self.get_model()
        # Fetch all records and check if new cloud is unique.
        # If not then determine if new one is up to date and store it.
        clouds = []
        unique = True
        found = None
        for row in model:
            found = row
            if row[0].id == cloud.id:
                unique = False
                # Merge comments
                for comm in cloud.comments:
                    if comm not in row[0].comments:
                        row[0].comments.append(comm)
                row[0].comments.sort(key=lambda r: r['created'])
                # Do we need to update status color?
                if row[0].updated < cloud.updated:
                    row[0].updated = cloud.updated
                    row[0].import_status(cloud.status)
                    #row[3] = cloud.color
                else:
                    pass
                break;

        if unique:
            # Get model and append a row with proper values
            tree_iter = model.append([cloud, cloud.page, cloud.author, cloud.color])
            #FIXME:self.par.current_iter should be deprecated
            self.par.current_iter = tree_iter
            #self.set_cursor(path)
            tree_path = self.get_model().get_path(tree_iter)
            self.set_cursor(tree_path)
        return self

    ###########################################################################
    def create_columns(self, tree):
        """Makes some columns for the list of clouds."""
        # Create cell renderer for text
        renderer = gtk.CellRendererText()
        # Create a column with proper renderer and point out row index
        # from which text should be taken.
        column = gtk.TreeViewColumn(_("#"),
                                    renderer,
                                    text=1,
                                    background_gdk=3)
        #column.set_attribute(renderer, )
        #column.set_alignment(1.0)
        # Set ID of a column that will be sorted when column header
        # was clicked. It also makes header clickable.
        column.set_sort_column_id(1)    
        # Add a column to the TreeView
        tree.append_column(column)
        
        # Create cell renderer for text
        renderer = gtk.CellRendererText()
        # Create a column with proper renderer and point out row index
        # from which text should be taken.
        column = gtk.TreeViewColumn(_('Committer'),
                                    renderer,
                                    text=2,
                                    background_gdk=3)
        
        # Set ID of a column that will be sorted when column header
        # was clicked. It also makes header clickable.
        column.set_sort_column_id(2)
        # Add a column to the TreeView
        tree.append_column(column)

    ###########################################################################
    def on_cursor_changed(self, widget):
        path, tree = widget.get_cursor()
        if path is not None:
            # Get model
            model = widget.get_model()
            # Unset highlight mode for all the clouds.
            # There is no trivial way to access certain cloud by
            # the path because it points to a position in TreeView list,
            # not in model list.
            for i in model:
                i[0].highlight = False
            # Get current cloud
            cloud = model[path][0]
            # Set highlight mode for current cloud to True
            cloud.highlight = True
            # Generate list of comments for the cloud
            self.par.comments.compose_comments(cloud)
            # Make "Add" button sensitive. It allows to add new comments
            self.par.add_comment_button.set_sensitive(True)
            # Set status control sensitive. It allows to set status
            self.par.cloud_controls.status.set_sensitive(True)
            # Set current status
            self.par.cloud_controls.status.set_active(cloud.status)
            #cr = self.par.par.view.window.cairo_create()
            self.par.par.view.queue_draw()

    ##########################################################################
    def on_activated(self, widget, row, col):
        """Fill the widget with list of cloud's comments."""
        # Get model of the list
        model = widget.get_model()
        # Get cloud object stored in the model
        cloud = model[row][0]
        #TODO: Add cursor state busy
        self.par.par.page_field.sb.set_value(cloud.page)
        #TODO: Add cursor state pointer (default)

# end of remarker.py
