#! /usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2010 Hannibal Team.
"""
Module that provides render functionality with Poppler(PDF) and Cairo(clouds)
"""
import gtk
import pango
import poppler
import cairo

import client
import cloud

from customs import RemarkGrabber
ONLINE_MODE = client.check_server()

# Scale constants
#SCALE_FIT_ON_PAGE = 0
#SCALE_FIT_WIDTH = 1
SCALE_100_PERCENT = 0
SCALE_200_PERCENT = 1
SCALE_300_PERCENT = 2

#FIXME: The DPI of the document should be fetched from the meta info
#       The DPI of the OS must be fetched as well
SCALE_FIX = 96.0/72.0           # scale correction


class View(gtk.DrawingArea):

    def __init__(self, parent=None):
        gtk.DrawingArea.__init__(self)
        self.par = parent       # parent widget
        self.buf = None         # pixbuf buffer that boosts up drawings
        self.sub_buf = None     # pixbuf subbuffer that boosts up drawings
        self.scale = 1          # scale
        self.file_uri = ''      # path to PDF
        self.document = None    # container for PDF
        self.n_pages = 0        # number of pages
        self.cur_page_num = 0   # current page number

        self.width = 0          # drawing area width
        self.height = 0         # drawing area height

        self.bpx = -1           # x-coord for mouse
        self.bpy = -1           # y-coord for mouse

        self.old_x = -1         # previous mouse x-coord
        self.old_y = -1         # previous mouse y-coord

        #self.image = gtk.gdk.pixbuf_new_from_file("logo.svg")

        # We want to take care of some events
        self.add_events(gtk.gdk.BUTTON_PRESS_MASK
                        | gtk.gdk.BUTTON_RELEASE_MASK
                        | gtk.gdk.POINTER_MOTION_MASK)

        # Binding events to according callbacks
        self.connect("realize", self.on_realize)
        self.connect("expose-event", self.on_expose)
        self.connect("button-press-event", self.on_button_press_event)
        self.connect("button-release-event", self.on_button_release_event)
        self.connect("motion-notify-event", self.on_motion_notify_event)

        self.set_flags(gtk.CAN_FOCUS)


    ##########################################################################
    # Callbacks
    ##########################################################################

    ##########################################################################
    def on_realize(self, event):
        """Callback for on-realize event."""
        self.gc = self.window.new_gc()

    ##########################################################################
    def on_button_press_event(self, widget, event):
        """Callback for mouse button press event.

        Just stores the first pair of coords of a cloud."""
        if self.document is None:
            return

        self.bpx = int(event.x)
        self.bpy = int(event.y)
        # Coords cannot be greater than page's size
        if self.bpx > self.width - 1 or self.bpy > self.height - 1:
            self.bpy = self.bpx = -1
            return
        # Coords cannot be negative
        if self.bpx < 2:
            self.bpx = -1
            self.bpy = -1
        if self.bpy < 2:
            self.bpx = -1
            self.bpy = -1

    ##########################################################################
    def on_button_release_event(self, widget, event):
        """Callback for mouse button release event

        Calculates the second pair of coords of a cloud."""
        
        if self.document is None or self.bpx == -1:
            return
        # Cast type to integer
        x = int(event.x)
        y = int(event.y)
        # Coords cannot be negative
        if x < 3:
            x = 3
        if y < 3:
            y = 3
        # Coords cannot be greater than page's size
        if x > self.width - 1:
            x = self.width - 2
        if y > self.height - 1:
            y = self.height - 2
        # Get width and height of new rectangle 
        dx = abs(self.bpx - x)
        dy = abs(self.bpy - y)
        # Clouds with size less than 10x10 are forbidden
        if dx < 10 or dy < 10:
            # Set bpx and bpy (button pressed x and y) to -1. It indicates
            # that there is no cloud at all
            self.bpx = self.bpy = -1
            # Render sub buffer if it exists to hide previous
            # temporary cloud
            if self.sub_buf is not None:
                self.window.draw_pixbuf(gc=self.gc, pixbuf=self.sub_buf,
                                        src_x=0,
                                        src_y=0,
                                        dest_x=self.old_x - 1,
                                        dest_y=self.old_y - 1)
                # Delete sub buffer
                del self.sub_buf
                self.sub_buf = None
            self.queue_draw()
            return

        # As we already have width(dx) and height(dy) of our new cloud
        # we are going to choose proper x and y coords. Mouse pointer
        # can be moved at any directions. Less coord is what we need
        if x > self.bpx:
            x = self.bpx
        if y > self.bpy:
            y = self.bpy
        # Finally, we have to scale down our coords. Cloud always 
        # scales itself, so it is enough to give it coords without any scale
        x = int(x / self.scale)
        y = int(y / self.scale)
        dx = int(dx / self.scale)
        dy = int(dy / self.scale)
        # Set bpx and bpy (button pressed x and y) to -1. It indicates
        # that there is no cloud at all
        self.bpx = self.bpy = - 1
        # Create new Rectangle with given params
        r = gtk.gdk.Rectangle(x, y, dx, dy)
        # Create new cloud and draw it on cairo context
        cr = self.window.cairo_create()
        #FIXME: don't use new cloud any time:
        # it would be better to optimize this process with global cloud bound
        # to View class and used to render new cloud. After cloud was created
        # View.cloud shoud be set to None.
        new_cloud = cloud.Cloud(r, self.cur_page_num)
        new_cloud.draw(cr, self.scale)
        # Run RemarkGrabber. It shows popup window with entry for remark text.
        # When <Enter> pressed it appends new cloud to the list.
        # When <Esc> pressed it destroys cloud.
        rg = RemarkGrabber(self.par, new_cloud)
        #new_cloud.draw(cr, self.scale)
        result = rg.run()
        rg.destroy()
        '''
        if result == gtk.RESPONSE_OK:
            # Render sub bufffer
            # and destroy it as we don't need it any more
            self.window.draw_pixbuf(gc=self.gc, pixbuf=self.sub_buf,
                                        src_x=0,
                                        src_y=0,
                                        dest_x=self.old_x - 2,
                                        dest_y=self.old_y - 2)
        '''
        del self.sub_buf
        self.sub_buf = None
            

        #self.queue_draw_area(int(x), int(y), int(dx), int(dy))

        if ONLINE_MODE:
            self.par.clouds = client.send_data(self.cur_page_num, t)
            self.put_buffer()
            self.refresh_recs()

        #self.refresh_recs()
        self.queue_draw()
            

    ##########################################################################
    def on_motion_notify_event(self, widget, event):
        """Callback for mouse pointer movement"""
        # Get out of here if no document is opened or no click performed
        if self.document is None or self.bpx == -1:
            return

        x = int(event.x)
        y = int(event.y)

        if x < 3:
            x = 3
        if y < 3:
            y = 3

        if x >= self.width - 1:
            x = self.width - 2
        if y >= self.height - 1:
            y = self.height - 2

        dx = abs(self.bpx - x)
        dy = abs(self.bpy - y)

        if x > self.bpx:
            x = self.bpx
        if y > self.bpy:
            y = self.bpy

        if self.sub_buf:
            self.window.draw_pixbuf(gc=self.gc, pixbuf=self.sub_buf,
                                    src_x=0,
                                    src_y=0,
                                    dest_x=self.old_x - 2,
                                    dest_y=self.old_y - 2)
            del self.sub_buf
            self.sub_buf = None
        # Remember surface under the cloud.
        self.sub_buf = self.buf.subpixbuf(src_x=int(x) - 2,
                                            src_y=int(y) - 2,
                                            width=int(dx) + 4,
                                            height=int(dy) + 4)
        self.old_x = int(x)
        self.old_y = int(y)

        x = x / self.scale
        y = y / self.scale
        dx = dx / self.scale
        dy = dy / self.scale
        cr = widget.window.cairo_create()
        cloud.Cloud(gtk.gdk.Rectangle(int(x), int(y), int(dx), int(dy)),\
            self.cur_page_num)\
            .draw(cr, self.scale)

    ##########################################################################
    def on_expose(self, widget, event):
        """Callback for expose event.

        Draws widget"""
        # Get cairo context
        self.cr = widget.window.cairo_create()
        # Change scale if needed
        if self.scale != 1:
            self.cr.scale(self.scale, self.scale)
        # Render PDF to pixbuf first
        if self.buf is None:
            self.get_buffer()
        # Render PDF form pixbuf to widget's surface
        self.put_buffer()
        # Draw clouds onto cairo context
        self.refresh_recs()

    ##########################################################################
    def refresh_recs(self):
        """Draw all clouds on current page"""
        clouds = self.par.rem.get_clouds(self.cur_page_num+1)
        cr = self.window.cairo_create()
        if clouds:
            for r in clouds:
                r.draw(cr, self.scale)

    ##########################################################################
    def get_buffer(self):
        if self.document is None:
            return
        # Get PDF page resolutions
        self.width, self.height = self.current_page.get_size()
        # Cast type to integer
        self.width = int(self.width * self.scale)
        self.height = int(self.height * self.scale)
        # Set proper resolutions for widget according to PDF page
        self.set_size_request(self.width, self.height)
        # Create a pixbuf
        self.buf = gtk.gdk.Pixbuf(colorspace=gtk.gdk.COLORSPACE_RGB,
                                has_alpha=False, bits_per_sample=8,
                                width=self.width,
                                height=self.height)
        # Render PDF page to pixbuf
        try:
            self.current_page.render_to_pixbuf(src_x=0, src_y=0,
                                src_width=self.width,
                                src_height=self.height, scale=self.scale,
                                rotation=0, pixbuf=self.buf)
        except:
            pass

    ##########################################################################
    def put_buffer(self):
        if self.buf is not None:
            # Render pixbuf with PDF content onto widget
            self.window.draw_pixbuf(gc=self.gc, pixbuf=self.buf,
                                    src_x=0, src_y=0, dest_x=0, dest_y=0)

    ##########################################################################
    def delete_event(self, widget, event, data=None):
        #close application and its window
        return False

    ##########################################################################
    def gtk_quit_event(self, widget, data=None):
        gtk.main_quit()

    ##########################################################################
    def destroy(self, widget, data=None):
        gtk.main_quit()

    ##########################################################################
    def set_scale(self, n):
        """Sets scale."""
        #print "scale"
        if      n == SCALE_100_PERCENT:
            self.scale = 1
        elif    n == SCALE_200_PERCENT:
            self.scale = 2
        elif    n == SCALE_300_PERCENT:
            self.scale = 3
        elif    n == SCALE_FIT_ON_PAGE:
            self.scale = 1
        elif    n == SCALE_FIT_WIDTH:
            self.scale = 1
        # Fix scale
        self.scale *= SCALE_FIX
        del self.buf
        self.buf = None
        self.queue_draw()


    ##########################################################################
    # Data
    ##########################################################################

    ##########################################################################
    def get_from_file(self, f):
        try:
            self.file_uri = "file://" + f
            self.document = poppler.document_new_from_file(self.file_uri,
                                                           None)
        except:
            print "[e] View.get_from_file"
            return

        self.n_pages = self.document.get_n_pages()
        self.cur_page_num = 0
        self.current_page = self.document.get_page(self.cur_page_num)

        del self.buf
        self.buf = None

        # Clear list of clouds
        self.par.rem.clear()
        # Now queue redraw
        self.queue_draw()
        self.par.toolbar.set_sensitive(True)
        # PDF info:
        self.par.document_title = str(self.document.get_property("title"))
        '''
        print "Title:", self.document.get_property("title")
        print "Format:", self.document.get_property("format")
        print "Format-major:", self.document.get_property("format-major")
        print "Format-minor:", self.document.get_property("format-minor")
        print "Author:", self.document.get_property("author")
        print "Subject:", self.document.get_property("subject")
        print "Keywords:", self.document.get_property("keywords")
        print "Creator:", self.document.get_property("creator")
        print "Producer:", self.document.get_property("producer")
        print "creation-date:", self.document.get_property("creation-date")
        print "mod-date:", self.document.get_property("mod-date")
        print "linearized:", self.document.get_property("linearized")
        print "page-layout:", self.document.get_property("page-layout")
        print "page-mode:", self.document.get_property("page-mode")
        print "viewer-preferences:", self.document.get_property("viewer-preferences")
        print "permissions:", self.document.get_property("permissions")
        print "metadata:", self.document.get_property("metadata")
        '''
        #self.set_size_request(int(self.width * self.scale),
                                #int(self.height * self.scale))

    ##########################################################################
    def set_page_num(self, num):
        self.current_page = self.document.get_page(num)
        self.cur_page_num = num

        self.width, self.height = self.current_page.get_size()
        self.width = int(self.width * self.scale)
        self.height = int(self.height * self.scale)
        self.set_size_request(self.width, self.height)
        self.buf = None
        self.queue_draw()

    ##########################################################################
    def next_page(self):
        if self.cur_page_num < self.n_pages-1:
            self.cur_page_num += 1
            self.set_page_num(self.cur_page_num)

    ##########################################################################
    def prev_page(self):
        if self.cur_page_num > 0:
            self.cur_page_num -= 1
            self.set_page_num(self.cur_page_num)

# end of view.py
